/***** BEGIN LICENSE BLOCK *****
    FlashGot - a Firefox extension for external download managers integration
    Copyright (C) 2004-2005 Giorgio Maone - g.maone@informaction.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

***** END LICENSE BLOCK *****/

const g_rxOpt=/^(inv|)opt-(.*)/;
g_downloadManagers=null;
g_opts=null;
g_extList=null;
g_extText=null;
g_referrerRadio=null;
g_referrerText=null;
g_tmpDir=null;

function flashgotOptions_onload() {
  
  if(g_flashGotService.mailer) { 
    // Thunderbird will handle "simple" clicks through Firefox
    document.getElementById("tab-downloads").setAttribute("collapsed","true");
  }
  
  g_downloadManagers=document.getElementById("downloadManagers");
  g_opts=document.getElementsByTagName("checkbox");
  
  const defaultDM=g_flashGotService.defaultDM; 
  const dms=g_flashGotService.DMS;
  var j,len,dm,menuItem;
  
  if(!dms.found) {
    g_downloadManagers.selectedItem=g_downloadManagers.appendItem("---",null);
    document.getElementById("nodms").removeAttribute("collapsed");
  } 
  for(j=0,len=dms.length; j<len; j++) {
    dm=dms[j];
    menuItem=g_downloadManagers.appendItem(dm.name,dm.codeName);
    if(dm.supported) {
      if(defaultDM==dm.name) {
        g_downloadManagers.selectedItem=menuItem;
      }
    } else {
      menuItem.setAttribute("disabled","true");
    }
  }
  
  g_extList=document.getElementById("ext-list");
  g_extText=document.getElementById("ext-text");
  g_extList.removeItemAt(0);
  var extensions=g_flashGotService.extensions;
  for(j=0, len=extensions.length; j<len; j++) {
    g_extList.appendItem(extensions[j],extensions[j]);
  }
  
  flashgotOptions_visitCheckboxes(
    function(prefName,inverse,checkbox) {
      var val=g_flashGotService.getPref(prefName);
      checkbox.checked=inverse?!val:val;
    }
  );
  
  g_referrerRadio=document.getElementById("referrer-radio");
  g_referrerText=document.getElementById("referrer-text");
  g_referrerRadio.selectedIndex=g_flashGotService.getPref("autoReferrer")?0:1;
  g_referrerText.value=g_flashGotService.getPref("fakeReferrer","");
  flashgotOptions_syncReferrer();
  g_referrerRadio.addEventListener("select", flashgotOptions_syncReferrer, true);
  
  flashgotOptions_syncDMOptions();
  g_downloadManagers.addEventListener("popuphidden", flashgotOptions_syncDMOptions, true);
  if(g_flashGotService.tmpDir) {
    document.getElementById("tmpdir-text").value=g_flashGotService.tmpDir.parent.path;
  }
  
  document.getElementById("interception-rg").selectedItem=
    document.getElementById("intercept"+
      (g_flashGotService.getPref("interceptAll")?"All":"Ext")+"-radio");
}

function flashgotOptions_interceptionSelected(rg) {
  document.getElementById('extensions-box').style.visibility=
    rg.selectedItem && rg.selectedItem.id=='interceptAll-radio' ? "hidden":"visible";
}

function flashgotOptions_syncDMOptions() {
  const dmrx=g_downloadManagers.value?
    new RegExp("\\b"+g_downloadManagers.value+"\\b")
    :null;
  const dmopts=document.getElementsByAttribute("class","dm-opt");
  var dmid,dmopt;
  for(var j=dmopts.length; j-->0;) {
    dmopt=dmopts[j];
    if(dmrx) {
      dmid=dmopt.id;
      if(dmid) {
        dmopt.setAttribute("collapsed", dmid.match(dmrx)?"false":"true" );
      }
    } else {
      dmopt.setAttribute("collapsed", "true" );
    }
  }
}

function flashgotOptions_syncReferrer() {
  if(g_referrerRadio.selectedIndex==1) {
    g_referrerText.removeAttribute("disabled");
  } else {
    g_referrerText.setAttribute("disabled","true");
  }
}
function flashgotOptions_enable(id,enabled) {
  var b=document.getElementById(id);
  if(enabled) {
    b.removeAttribute("disabled");
  } else {
     b.setAttribute("disabled","true");
  }
}

function flashgotOptions_extText_changed() {
  var enable;
  var value=g_extText.value;
  if((!g_extText.disabled) &&
    /[\w\-]+/.test(value)) {
      enable=true;
      for(var j=g_extList.getRowCount();
          j-- >0 && (enable=g_extList.getItemAtIndex(j).value!=value)
          ;);
  } else {
     enable=false;
  }
  flashgotOptions_enable("ext-add-button", enable);
}

function flashgotOptions_extList_changed() {
  flashgotOptions_enable("ext-remove-button",
    g_extList.selectedCount>0);
  flashgotOptions_extText_changed();
}

function flashgotOptions_ext_add() {
  g_extList.appendItem(g_extText.value,g_extText.value);
  flashgotOptions_extText_changed();
}

function flashgotOptions_ext_remove() {
  const selectedItems=g_extList.selectedItems;
  for(var j=selectedItems.length; j-->0;) {
    g_extList.removeItemAt(g_extList.getIndexOfItem(selectedItems[j]));
  }
}
function flashgotOptions_save() {
  if(g_downloadManagers.selectedItem && g_downloadManagers.selectedItem.getAttribute("label")) {
    g_flashGotService.defaultDM=g_downloadManagers.selectedItem.getAttribute("label");
  }
  flashgotOptions_visitCheckboxes(
    function(prefName,inverse,checkbox) {
      g_flashGotService.setPref(prefName,inverse?!checkbox.checked:checkbox.checked);
    }
  );
  
  const extensions=[];
  var j;
  for(j=g_extList.getRowCount(); j-->0;) {
    extensions[extensions.length]=g_extList.getItemAtIndex(j).value;
  }
  g_flashGotService.extensions=extensions;
  g_flashGotService.setPref("autoReferrer",g_referrerRadio.value=="true");
  g_flashGotService.setPref("fakeReferrer",g_referrerText.value);
  if(g_tmpDir) g_flashGotService.globals.prefs.setComplexValue("tmpDir", Components.interfaces.nsILocalFile, g_tmpDir);
  g_flashGotService.setPref("interceptAll",document.getElementById("interceptAll-radio").selected);
  return true;
  
}

function flashgotOptions_visitCheckboxes(callback) {
  var j,checkbox,match;
  for(j=g_opts.length; j-->0;) {
    checkbox=g_opts[j];
    if(match=checkbox.id.match(g_rxOpt)) {
      callback(match[2],match[1]=="inv",checkbox);
    }
  }
}


function flashgotOptions_showLog() {
  try {
    window.opener.open(Components.classes['@mozilla.org/network/io-service;1'
      ].getService(Components.interfaces.nsIIOService
      ).newFileURI(g_flashGotService.logFile).spec, "_blank",
      "scrollbars=yes,menubar=yes,titlebar=yes,resizable=yes");
  } catch(ex) { dump(ex.message); }
}


function flashgotOptions_clearLog() {
  g_flashGotService.clearLog();
}

function flashgotOptions_browseTmpDir() {
  const cc=Components.classes;
  const ci=Components.interfaces;
  const fp = cc["@mozilla.org/filepicker;1"].createInstance(ci.nsIFilePicker);
  const title="FlashGot - "+document.getElementById("tmpdir-label").value;
  fp.init(window, title, ci.nsIFilePicker.modeGetFolder);
  try {
    fp.displayDirectory = g_tmpDir==null?g_flashGotService.tmpDir.parent:g_tmpDir;
  } catch (ex) { g_flashGotService.log(ex); }
  fp.appendFilters(ci.nsIFilePicker.filterAll);
  if (fp.show()==ci.nsIFilePicker.returnOK) {
    g_tmpDir=fp.file.QueryInterface(ci.nsILocalFile);
    document.getElementById("tmpdir-text").value=g_tmpDir.path;
    document.getElementById("tmpdir-warning").style.visibility="visible";
  }
}

